package sun.java2d.opengl;

import sun.java2d.loops.GraphicsPrimitive;
import java.security.AccessController;
import sun.security.action.GetPropertyAction;
import java.awt.GraphicsEnvironment;
import sun.awt.image.PixelConverter;
import sun.java2d.pipe.BufferedContext;
import java.awt.Rectangle;
import sun.java2d.pipe.RenderBuffer;
import sun.java2d.loops.MaskFill;
import sun.java2d.loops.CompositeType;
import java.awt.AlphaComposite;
import java.awt.Composite;
import sun.java2d.SunGraphics2D;
import java.awt.image.Raster;
import sun.java2d.SurfaceDataProxy;
import java.awt.image.ColorModel;
import sun.java2d.pipe.ParallelogramPipe;
import sun.java2d.pipe.PixelToParallelogramConverter;
import sun.java2d.loops.SurfaceType;
import sun.java2d.pipe.hw.AccelSurface;
import sun.java2d.SurfaceData;

public abstract class OGLSurfaceData extends SurfaceData implements AccelSurface
{
    public static final int FBOBJECT = 5;
    public static final int PF_INT_ARGB = 0;
    public static final int PF_INT_ARGB_PRE = 1;
    public static final int PF_INT_RGB = 2;
    public static final int PF_INT_RGBX = 3;
    public static final int PF_INT_BGR = 4;
    public static final int PF_INT_BGRX = 5;
    public static final int PF_USHORT_565_RGB = 6;
    public static final int PF_USHORT_555_RGB = 7;
    public static final int PF_USHORT_555_RGBX = 8;
    public static final int PF_BYTE_GRAY = 9;
    public static final int PF_USHORT_GRAY = 10;
    public static final int PF_3BYTE_BGR = 11;
    private static final String DESC_OPENGL_SURFACE = "OpenGL Surface";
    private static final String DESC_OPENGL_SURFACE_RTT = "OpenGL Surface (render-to-texture)";
    private static final String DESC_OPENGL_TEXTURE = "OpenGL Texture";
    static final SurfaceType OpenGLSurface;
    static final SurfaceType OpenGLSurfaceRTT;
    static final SurfaceType OpenGLTexture;
    private static boolean isFBObjectEnabled;
    private static boolean isLCDShaderEnabled;
    private static boolean isBIOpShaderEnabled;
    private static boolean isGradShaderEnabled;
    private OGLGraphicsConfig graphicsConfig;
    protected int type;
    private int nativeWidth;
    private int nativeHeight;
    protected static OGLRenderer oglRenderPipe;
    protected static PixelToParallelogramConverter oglTxRenderPipe;
    protected static ParallelogramPipe oglAAPgramPipe;
    protected static OGLTextRenderer oglTextPipe;
    protected static OGLDrawImage oglImagePipe;
    
    protected native boolean initTexture(long p0, boolean p1, boolean p2, boolean p3, int p4, int p5);
    
    protected native boolean initFBObject(long p0, boolean p1, boolean p2, boolean p3, int p4, int p5);
    
    protected native boolean initFlipBackbuffer(long p0);
    
    private native int getTextureTarget(long p0);
    
    private native int getTextureID(long p0);
    
    protected OGLSurfaceData(OGLGraphicsConfig graphicsConfig, ColorModel cm, int type) {
        super(getCustomSurfaceType(type), cm);
        this.graphicsConfig = graphicsConfig;
        this.type = type;
        this.setBlitProxyKey(graphicsConfig.getProxyKey());
    }
    
    @Override
    public SurfaceDataProxy makeProxyFor(SurfaceData surfaceData) {
        return OGLSurfaceDataProxy.createProxy(surfaceData, this.graphicsConfig);
    }
    
    private static SurfaceType getCustomSurfaceType(int n) {
        switch (n) {
            case 3: {
                return OGLSurfaceData.OpenGLTexture;
            }
            case 5: {
                return OGLSurfaceData.OpenGLSurfaceRTT;
            }
            default: {
                return OGLSurfaceData.OpenGLSurface;
            }
        }
    }
    
    private void initSurfaceNow(int n, int n2) {
        int n3 = this.getTransparency() == 1;
        boolean n4 = false;
        switch (this.type) {
            case 3: {
                n4 = this.initTexture(this.getNativeOps(), (boolean)n3, this.isTexNonPow2Available(), this.isTexRectAvailable(), n, n2);
                break;
            }
            case 5: {
                n4 = this.initFBObject(this.getNativeOps(), (boolean)n3, this.isTexNonPow2Available(), this.isTexRectAvailable(), n, n2);
                break;
            }
            case 4: {
                n4 = this.initFlipBackbuffer(this.getNativeOps());
                break;
            }
        }
        if (n4 == 0) {
            throw new OutOfMemoryError("can't create offscreen surface");
        }
        return;
    }
    
    protected void initSurface(final int n, final int n2) {
        OGLRenderQueue instance = OGLRenderQueue.getInstance();
        instance.lock();
        try {
            switch (this.type) {
                case 3:
                case 5: {
                    OGLContext.setScratchSurface(this.graphicsConfig);
                    break;
                }
            }
            instance.flushAndInvokeNow(new Runnable() {
                @Override
                public void run() {
                    OGLSurfaceData.access$000(OGLSurfaceData.this, n, n2);
                }
            });
        }
        finally {
            instance.unlock();
        }
    }
    
    @Override
    public final OGLContext getContext() {
        return this.graphicsConfig.getContext();
    }
    
    final OGLGraphicsConfig getOGLGraphicsConfig() {
        return this.graphicsConfig;
    }
    
    @Override
    public final int getType() {
        return this.type;
    }
    
    public final int getTextureTarget() {
        return this.getTextureTarget(this.getNativeOps());
    }
    
    public final int getTextureID() {
        return this.getTextureID(this.getNativeOps());
    }
    
    @Override
    public long getNativeResource(int n) {
        if (n == 3) {
            return (long)this.getTextureID();
        }
        return 0L;
    }
    
    @Override
    public Raster getRaster(int n, int n2, int n3, int n4) {
        throw new InternalError("not implemented yet");
    }
    
    @Override
    public boolean canRenderLCDText(SunGraphics2D sunGraphics2D) {
        return this.graphicsConfig.isCapPresent(131072) && sunGraphics2D.surfaceData.getTransparency() == 1 && sunGraphics2D.paintState <= 0 && (sunGraphics2D.compositeState <= 0 || (sunGraphics2D.compositeState <= 1 && this.canHandleComposite(sunGraphics2D.composite)));
    }
    
    private boolean canHandleComposite(Composite composite) {
        if ((composite instanceof AlphaComposite)) {
            AlphaComposite alphaComposite = (AlphaComposite)composite;
            return alphaComposite.getRule() == 3 && alphaComposite.getAlpha() >= 1.0f;
        }
        return false;
    }
    
    @Override
    public void validatePipe(SunGraphics2D sg2d) {
        int n = 0;
        OGLTextRenderer textpipe;
        if ((sg2d.compositeState <= 0 && sg2d.paintState <= 1) || (sg2d.compositeState == 1 && sg2d.paintState <= 1 && ((AlphaComposite)sg2d.composite).getRule() == 3) || (sg2d.compositeState == 2 && sg2d.paintState <= 1)) {
            textpipe = OGLSurfaceData.oglTextPipe;
        }
        else {
            super.validatePipe(sg2d);
            textpipe = sg2d.textpipe;
            n = 1;
        }
        PixelToParallelogramConverter pixelToParallelogramConverter = null;
        OGLRenderer fillpipe = null;
        if (sg2d.antialiasHint != 2) {
            if (sg2d.paintState <= 1) {
                if (sg2d.compositeState <= 2) {
                    pixelToParallelogramConverter = OGLSurfaceData.oglTxRenderPipe;
                    fillpipe = OGLSurfaceData.oglRenderPipe;
                }
            }
            else if (sg2d.compositeState <= 1 && OGLPaints.isValid(sg2d)) {
                pixelToParallelogramConverter = OGLSurfaceData.oglTxRenderPipe;
                fillpipe = OGLSurfaceData.oglRenderPipe;
            }
        }
        else if (sg2d.paintState <= 1) {
            if (this.graphicsConfig.isCapPresent(256) && (sg2d.imageComp == CompositeType.SrcOverNoEa || sg2d.imageComp == CompositeType.SrcOver)) {
                if (n == 0) {
                    super.validatePipe(sg2d);
                    n = 1;
                }
                PixelToParallelogramConverter shapepipe = new PixelToParallelogramConverter(sg2d.shapepipe, OGLSurfaceData.oglAAPgramPipe, 0.125, 0.499, false);
                sg2d.drawpipe = shapepipe;
                sg2d.fillpipe = shapepipe;
                sg2d.shapepipe = shapepipe;
            }
            else if (sg2d.compositeState == 2) {
                pixelToParallelogramConverter = OGLSurfaceData.oglTxRenderPipe;
                fillpipe = OGLSurfaceData.oglRenderPipe;
            }
        }
        if (pixelToParallelogramConverter != null) {
            if (sg2d.transformState >= 3) {
                sg2d.drawpipe = pixelToParallelogramConverter;
                sg2d.fillpipe = pixelToParallelogramConverter;
            }
            else if (sg2d.strokeState != 0) {
                sg2d.drawpipe = pixelToParallelogramConverter;
                sg2d.fillpipe = fillpipe;
            }
            else {
                sg2d.drawpipe = fillpipe;
                sg2d.fillpipe = fillpipe;
            }
            sg2d.shapepipe = pixelToParallelogramConverter;
        }
        else if (n == 0) {
            super.validatePipe(sg2d);
        }
        sg2d.textpipe = textpipe;
        sg2d.imagepipe = OGLSurfaceData.oglImagePipe;
    }
    
    @Override
    protected MaskFill getMaskFill(SunGraphics2D sg2d) {
        if (sg2d.paintState > 1 && (!OGLPaints.isValid(sg2d) || !this.graphicsConfig.isCapPresent(16))) {
            return null;
        }
        return super.getMaskFill(sg2d);
    }
    
    @Override
    public boolean copyArea(SunGraphics2D sunGraphics2D, int n, int n2, int n3, int n4, int n5, int n6) {
        if (sunGraphics2D.transformState < 3 && sunGraphics2D.compositeState < 2) {
            n += sunGraphics2D.transX;
            n2 += sunGraphics2D.transY;
            OGLSurfaceData.oglRenderPipe.copyArea(sunGraphics2D, n, n2, n3, n4, n5, n6);
            return true;
        }
        return false;
    }
    
    @Override
    public void flush() {
        this.invalidate();
        OGLRenderQueue instance = OGLRenderQueue.getInstance();
        instance.lock();
        try {
            OGLContext.setScratchSurface(this.graphicsConfig);
            RenderBuffer buffer = instance.getBuffer();
            instance.ensureCapacityAndAlignment(12, 4);
            buffer.putInt(72);
            buffer.putLong(this.getNativeOps());
            instance.flushNow();
        }
        finally {
            instance.unlock();
        }
    }
    
    static void dispose(long x, OGLGraphicsConfig scratchSurface) {
        OGLRenderQueue instance = OGLRenderQueue.getInstance();
        instance.lock();
        try {
            OGLContext.setScratchSurface(scratchSurface);
            RenderBuffer buffer = instance.getBuffer();
            instance.ensureCapacityAndAlignment(12, 4);
            buffer.putInt(73);
            buffer.putLong(x);
            instance.flushNow();
        }
        finally {
            instance.unlock();
        }
    }
    
    static void swapBuffers(long x) {
        OGLRenderQueue instance = OGLRenderQueue.getInstance();
        instance.lock();
        try {
            RenderBuffer buffer = instance.getBuffer();
            instance.ensureCapacityAndAlignment(12, 4);
            buffer.putInt(80);
            buffer.putLong(x);
            instance.flushNow();
        }
        finally {
            instance.unlock();
        }
    }
    
    boolean isTexNonPow2Available() {
        return this.graphicsConfig.isCapPresent(32);
    }
    
    boolean isTexRectAvailable() {
        return this.graphicsConfig.isCapPresent(1048576);
    }
    
    @Override
    public Rectangle getNativeBounds() {
        OGLRenderQueue instance = OGLRenderQueue.getInstance();
        instance.lock();
        try {
            return new Rectangle(this.nativeWidth, this.nativeHeight);
        }
        finally {
            instance.unlock();
        }
    }
    
    boolean isOnScreen() {
        return this.getType() == 1;
    }
    
    static void access$000(OGLSurfaceData oglSurfaceData, int n, int n2) {
        oglSurfaceData.initSurfaceNow(n, n2);
    }
    
    static {
        OpenGLSurface = SurfaceType.Any.deriveSubType("OpenGL Surface", PixelConverter.ArgbPre.instance);
        OpenGLSurfaceRTT = OGLSurfaceData.OpenGLSurface.deriveSubType("OpenGL Surface (render-to-texture)");
        OpenGLTexture = SurfaceType.Any.deriveSubType("OpenGL Texture");
        if (!GraphicsEnvironment.isHeadless()) {
            OGLSurfaceData.isFBObjectEnabled = !"false".equals((String)AccessController.doPrivileged(new GetPropertyAction("sun.java2d.opengl.fbobject")));
            OGLSurfaceData.isLCDShaderEnabled = !"false".equals((String)AccessController.doPrivileged(new GetPropertyAction("sun.java2d.opengl.lcdshader")));
            OGLSurfaceData.isBIOpShaderEnabled = !"false".equals((String)AccessController.doPrivileged(new GetPropertyAction("sun.java2d.opengl.biopshader")));
            OGLSurfaceData.isGradShaderEnabled = !"false".equals((String)AccessController.doPrivileged(new GetPropertyAction("sun.java2d.opengl.gradshader")));
            OGLRenderQueue instance = OGLRenderQueue.getInstance();
            OGLSurfaceData.oglImagePipe = new OGLDrawImage();
            OGLSurfaceData.oglTextPipe = new OGLTextRenderer(instance);
            OGLSurfaceData.oglRenderPipe = new OGLRenderer(instance);
            if (GraphicsPrimitive.tracingEnabled()) {
                OGLSurfaceData.oglTextPipe = OGLSurfaceData.oglTextPipe.traceWrap();
            }
            OGLSurfaceData.oglAAPgramPipe = OGLSurfaceData.oglRenderPipe.getAAParallelogramPipe();
            OGLSurfaceData.oglTxRenderPipe = new PixelToParallelogramConverter(OGLSurfaceData.oglRenderPipe, OGLSurfaceData.oglRenderPipe, 1.0, 0.25, true);
            OGLBlitLoops.register();
            OGLMaskFill.register();
            OGLMaskBlit.register();
        }
    }
}
